
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import org.xml.sax.*;

public class XBELContentHandler extends SAXTracker {
  protected BookmarkCollection bookmarks;
  protected Map aliasRefs;

  protected Node currentNode;

  public XBELContentHandler(String uri) {
    bookmarks = new BookmarkCollection(uri);
    keepContentsOf("title");
    keepContentsOf("desc");
    currentNode = bookmarks;
  }

  public BookmarkCollection getBookmarkCollection() {
    return bookmarks;
  }


  public void startElement(String nsuri, String lname, String qname,
			   Attributes attrs) throws SAXException {
    if (lname.equals("folder"))
      currentNode = new Folder(currentNode, attrs.getValue("id"));
    else if (lname.equals("bookmark"))
      currentNode = new Bookmark(currentNode, attrs.getValue("id"));
    else if (lname.equals("separator"))
      currentNode = new Separator(currentNode);
    else if (lname.equals("alias")) {
      currentNode = new Alias(currentNode);
      aliasRefs.put(currentNode, attrs.getValue("ref"));
    }

    if (lname.equals("folder") || lname.equals("bookmark") ||
	lname.equals("alias") || lname.equals("separator")) {
      NodeWithChildren parent = (NodeWithChildren) currentNode.getParent();
      parent.addChild(currentNode);

      if (lname.equals("folder") || lname.equals("bookmark"))
	bookmarks.registerNode((NodeWithChildren) currentNode);
    }
    
    super.startElement(nsuri, lname, qname, attrs);
  }
  
  public void endElement(String nsuri, String lname, String qname)
    throws SAXException {
    if (lname.equals("title"))
      ((NodeWithTitle) currentNode).setTitle(content.toString());
    else if (lname.equals("desc"))
      ((NodeWithTitle) currentNode).setDescription(content.toString());

    else if (lname.equals("folder") || lname.equals("bookmark") ||
	     lname.equals("alias") || lname.equals("separator"))
      currentNode = currentNode.getParent();    
    
    super.endElement(nsuri, lname, qname);
  }

  public void endDocument() {
    Iterator it = aliasRefs.keySet().iterator();
    while (it.hasNext()) {
      Alias alias = (Alias) it.next();
      String id = (String) aliasRefs.get(alias);
      alias.setNode(bookmarks.getNodeById(id));
    }
  }
}

