
import sys, string
from xml.dom.minidom import parse
from domutils import *

# --- Templates

top = \
"""
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<HTML>
<HEAD>
  <TITLE>%s</TITLE>
  <LINK REL=stylesheet HREF="play.css" TYPE="text/css">
</HEAD>

<BODY>

<H1>%s</H1>
"""

bottom = \
"""
<HR>

<ADDRESS>
Converted from Jon Bosak's XML to HTML with the DOM.
</ADDRESS>

</BODY>
</HTML>
"""

# --- Conversion functions

def convert(doc, out = None):
    out = out or sys.stdout
    
    play = doc.documentElement
    title = data(find_first_child(play, ELEMENT_NODE, "TITLE"),
                 "[untitled]")
    out.write(top % (title, title))

    _convert_front_matter(find_first_child(play, ELEMENT_NODE, "FM"), out)
    _convert_personae(find_first_child(play, ELEMENT_NODE, "PERSONAE"), out)

    for act in filter(predicate(ELEMENT_NODE, "ACT"), play.childNodes):
        _convert_act(act, out)
    
    out.write(bottom)

def _convert_front_matter(fm, out):
    if fm == None:
        return

    out.write("<h2>Front matter</h2>\n")
    for p in filter(predicate(ELEMENT_NODE, "P"), fm.childNodes):
        out.write("<p>%s</p>\n" % data(p))

def _convert_personae(personae, out):
    if personae == None:
        return

    out.write("<h2>%s</h2>" %
              data(find_first_child(personae, ELEMENT_NODE, "TITLE"),
                    "Dramatis Personae"))

    out.write("<ul>\n")
    for item in filter(type_predicate(ELEMENT_NODE), personae.childNodes):
        if item.nodeName == "PERSONA":
            out.write("  <li>%s</li>\n" % data(item))
        elif item.nodeName == "PGROUP":
            elems = filter(type_predicate(ELEMENT_NODE), item.childNodes)
            out.write("  <li>%s</li>\n" %
                      string.join(map(data, elems), ", "))
    
    out.write("</ul>\n")

def _convert_act(act, out):
    out.write("<h2>%s</h2>\n" %
              data(find_first_child(act, ELEMENT_NODE, "TITLE")))

    _convert_content(find_first_child(act, ELEMENT_NODE, "PROLOGUE"),
                     out)
    
    for scene in filter(predicate(ELEMENT_NODE, "SCENE"), act.childNodes):
        _convert_content(scene, out)

    _convert_content(find_first_child(act, ELEMENT_NODE, "EPILOGUE"),
                     out)

    
def _convert_content(container, out):
    if container == None:
        return
    
    out.write("<h3>%s</h3>\n" %
              data(find_first_child(container, ELEMENT_NODE, "TITLE"),
                   container.nodeName))

    for elem in filter(type_predicate(ELEMENT_NODE),
                       container.childNodes):
        if elem.nodeName == "SPEECH":
            out.write("<p class=speaker>%s</p>\n" %
                      data(find_first_child(elem, ELEMENT_NODE,
                                            "SPEAKER")))
            for line in filter(predicate(ELEMENT_NODE, "LINE"),
                               elem.childNodes):
                out.write("<p class=line>")
                for item in line.childNodes:
                    if item.nodeType == TEXT_NODE:
                        out.write(item.nodeValue)
                    elif item.nodeType == ELEMENT_NODE and \
                         item.nodeName == "STAGEDIR":
                        out.write("<i>%s</i>" % data(item))
                out.write("</p>\n")
                
        elif elem.nodeName == "STAGEDIR":
            out.write("<p class=stagedir>%s</p>\n" % data(elem))
    
# --- Main program

if len(sys.argv) == 3:
    outf = open(sys.argv[2], "w")
else:
    outf = sys.stdout
    
convert(parse(sys.argv[1]), outf)
