"""
This program converts an RSS document to an HTML document, using
xmllib.
"""

import xmllib

# --- Templates

top = \
"""
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<HTML>
<HEAD>
  <TITLE>%s</TITLE>
</HEAD>

<BODY>

<H1><A HREF="%s">%s</A></H1>
"""

bottom = \
"""
<HR>

<ADDRESS>
Converted to HTML, by xmllib_rss2html.py.
</ADDRESS>

</BODY>
</HTML>
"""

# --- The converter

class RSS2HTML(xmllib.XMLParser):

    def __init__(self, out):
        self.elements = {
            "rss"         : (None,               self.end_rss),
            "channel"     : (None,               self.end_channel),
            "item"        : (self.start_item,    self.end_item),
            "title"       : (self.start_pcdata,  self.end_title),
            "link"        : (self.start_pcdata,  self.end_link),
            "description" : (self.start_pcdata,  self.end_descr)}
        xmllib.XMLParser.__init__(self)
        self._out = out

        # state tracking variables
        self._data = ""
        self._item_count = 0
        
        self._title = None
        self._link = None
        self._descr = ''


    def handle_data(self, data):
        self._data = self._data + data

    def handle_cdata(self, data):
        self.handle_data(data)


    def end_title(self):
        self._title = self._data

    def end_link(self):
        self._link = self._data

    def end_descr(self):
        self._descr = self._data


    def end_channel(self):
        self._out.write(top % (self._title, self._link, self._title))
        
        if self._descr != None:
            self._out.write("<p>%s</p>" % self._descr)


    def start_item(self, atts):
        self._item_count = self._item_count + 1
        if self._item_count == 1:
            self.end_channel()
            self._out.write("\n<ul>\n")


    def end_item(self):
        self._out.write('  <li><a href="%s">%s</a> %s\n' %
                        (self._link, self._title, self._descr))
        self._descr = ''


    def start_pcdata(self, atts):
	self._data = ""        


    def end_rss(self):
        self._out.write("</ul>\n")
        self._out.write(bottom)


# --- Conversion driver

def convert(inf, out):
    p = RSS2HTML(out)

    buf = inf.read(16384)
    while buf != "":
        p.feed(buf)
        buf = inf.read(16384)

    p.close()

# --- The main program

import sys

inf = open(sys.argv[1])
outf = open(sys.argv[2], "w")

convert(inf, outf)

inf.close()
outf.close()
