"""
This application uses xmlproc to convert RSS documents to HTML.

$Id: xmlproc_rss2html.py,v 1.5 2001/08/19 19:27:41 larsga Exp $
"""

import sys
from xml.parsers.xmlproc import xmlproc, xmlapp

# --- Templates

top = \
"""
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<HTML>
<HEAD>
  <TITLE>%s</TITLE>
</HEAD>

<BODY>

<H1><A HREF="%s">%s</A></H1>
"""

bottom = \
"""
</UL>
<HR>

<ADDRESS>
Converted to HTML, by xmlproc_rss2html.py.
</ADDRESS>

</BODY>
</HTML>
"""


# --- The converter

class RSS2HTML(xmlapp.Application):

    def __init__(self, out = None):
        xmlapp.Application.__init__(self)
        self._out = out or sys.stdout
        
        # tracking state
        self._data = ""
        self._first_item = 1

        self._title = None
        self._link = None
        self._descr = None


    def handle_data(self, data, start, end):
        self._data = self._data + data[start : end]


    def handle_start_tag(self, name, attrs):
        self._data = ""

        if name == "item":
            self._descr = "" # reset for this item
            if self._first_item:
                self._out.write(top % (self._title, self._link,
                                       self._title))

                if self._descr:
                    self._out.write("<p>%s</p>" % self._descr)
                self._out.write("\n<UL>\n")

            self._first_item = 0

            
    def handle_end_tag(self, name):
        if name == "title":
            self._title = self._data

        elif name == "link":
            self._link = self._data

        elif name == "description":
            self._descr = self._data

        elif name == "item":
            self._out.write('  <li><a href="%s">%s</a> %s\n' %
                           (self._link, self._title, self._descr))

            
    def doc_end(self):
        self._out.write(bottom)

        
# --- The error handler

class ErrorPrinter(xmlapp.ErrorHandler):
    """This handler simply prints error messages to standard error."""

    def __init__(self, doc, out = None):
        self._out = out or sys.stderr
        self._doc = doc

    def warning(self,msg):
        self.__write("WARNING", msg)

    def error(self,msg):
        self.__write("ERROR", msg)
    
    def fatal(self,msg):
        self.__write("FATAL ERROR", msg)

        
    def __write(self, label, msg):
        location = "%s:%d:%d" % (self._locator.get_current_sysid(),
                                 self._locator.get_line(),
                                 self._locator.get_column())
        msg = "%s: %s at %s\n" % (label, msg, location)
        self._out.write(msg)
        self._doc.write("<PRE>%s</PRE>" % msg)        

        
# --- The driver

def convert(sysid, out):
    p = xmlproc.XMLProcessor()
    p.set_application(RSS2HTML(out))
    p.set_error_handler(ErrorPrinter(out))
    p.parse_resource(sysid)

    
# --- The main program

sysid = sys.argv[1]
out   = open(sys.argv[2], "w")

convert(sysid, out)

out.close()
