
import sys
from pyxie import xDispatch, File2PYX, PYX2xTree, PYXDecoder

# --- HTML templates

top = \
"""
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<HTML>
<HEAD>
  <TITLE>%s</TITLE>
  <LINK REL=stylesheet HREF="play.css" TYPE="text/css">
</HEAD>

<BODY>

<H1>%s</H1>
"""

bottom = \
"""
<HR>

<ADDRESS>
Converted from Jon Bosak's XML to HTML with Pyxie.
</ADDRESS>

</BODY>
</HTML>
"""

# --- Utilities

def find_right(tree, elem_name):
    while tree.HasRight() and not tree.AtElement(elem_name):
        tree.Right()

# --- The xDispatch application

class PlayHandler(xDispatch):

    def __init__(self, out = None):
        xDispatch.__init__(self)
        self._out = out or sys.stdout
        self._data = ""

    def characters(self, data):
        self._data = self._data + PYXDecoder(data)

    def default_start(self, name, attrs):
        self._data = ""

        
    def end_TITLE(self, name):
        parent = self.Ancestors[-1][0]
        
        if parent == "PLAY":
            self._out.write(top % (self._data, self._data))
            
        elif parent == "PERSONAE":
            self._out.write("<h2>%s</h2>\n" % self._data)
            self._out.write("<ul>\n")

        elif parent == "ACT":
            self._out.write("<h2>%s</h2>\n" % self._data)

        elif parent == "SCENE":
            self._out.write("<h3>%s</h3>\n" % self._data)
            
    def end_P(self, name):
        self._out.write("<p>%s</p>\n" % self._data)

    def end_PERSONAE(self, name):
        self._out.write("</ul>\n")

    def end_PERSONA(self, name):
        self._out.write("  <li>%s\n" % self._data)

        
    def start_PGROUP(self, name, attrs):
        self.PushElement(name, attrs)
        cur = PYX2xTree(self)

        persons = []
        descr = None

        cur.Down()
        while cur.HasRight():
            if cur.AtElement("PERSONA"):
                persons.append(cur.JoinData(""))
            elif cur.AtElement("GRPDESCR"):
                descr = cur.JoinData("")

            cur.Right()

        cur.ZapTree()
            
        self._out.write("  <li><b>%s</b>\n" % descr[ : -1])
        self._out.write("  <ul>\n")
        for person in persons:
            self._out.write("  <li>%s\n" % person)
        self._out.write("  </ul>\n")


    def end_STAGEDIR(self, name):
        self._out.write('<p class="stagedir">%s</p>\n' % self._data)
            
    def end_SPEAKER(self, name):
        self._out.write('<p class="speaker">%s</p>\n' % self._data)

    def end_PLAY(self, name):
        self._out.write(bottom)

        
    def start_LINE(self, name, attrs):
        self.PushElement(name, attrs)
        cur = PYX2xTree(self)
        cur.Down()

        self._out.write('<p class="line">')
        while 1:
            if cur.AtElement("STAGEDIR"):
                self._out.write("<i>%s</i>" % cur.JoinData(""))
            else:
                self._out.write(cur.Data)

            if cur.HasRight():
                cur.Right()
            else:
                break

        self._out.write("</p>\n")
        cur.ZapTree()

        
# --- Main program
        
pyxfile = File2PYX(sys.argv[1])
handler = PlayHandler()
handler.Dispatch(pyxfile)
