
import sys, codecs
from pyxie import xDispatch, File2PYX, PYXDecoder

# --- Templates

top = \
"""
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<HTML>
<HEAD>
  <TITLE>%s</TITLE>
</HEAD>

<BODY>
<A HREF="%s"><H1>%s</H1></A>
"""

bottom = \
"""
</UL>

<HR>

<ADDRESS>
Converted to HTML using Pyxie.
</ADDRESS>

</BODY>
</HTML>
"""

# --- xDispatch application

class RSS2HTML(xDispatch):

    def __init__(self, out = None):
        xDispatch.__init__(self)
        self._out = out or sys.stdout
        
        self._title = None
        self._link = None
        self._descr = None
        self._top_written = 0

    def characters(self, data):
        parent = self.Ancestors[-1][0]

        if parent == "title":
            self._title = PYXDecoder(data)
        elif parent == "link":
            self._link = PYXDecoder(data)
        elif parent == "description":
            self._descr = PYXDecoder(data)

            
    def start_item(self, name, attrs):
        if not self._top_written:
            self._out.write(top %
                            (self._title, self._link, self._title))
            if self._descr:
                self._out.write("<p>%s</p>\n" % self._descr)
            self._out.write("<ul>\n")
            self._top_written = 1

        self._title = None
        self._link = None
        self._descr = ""

    def end_item(self, name):
        self._out.write('<li><a href="%s">%s</a> %s\n' %
                        (self._link, self._title, self._descr))

    def end_rss(self, name):
        self._out.write(bottom)
        
# --- Main program

pyxfile = File2PYX(sys.argv[1])

handler = RSS2HTML(codecs.open(sys.argv[2], "w", "iso-8859-1"))
handler.Dispatch(pyxfile)
