"""Classes to simplify the development of RSS clients.

$Id: clientlib.py,v 1.2 2001/01/12 08:16:53 larsga Exp $
"""

try:
    import cPickle
    pickle = cPickle
except ImportError:
    import pickle


import time
from xml.sax._exceptions import SAXException
import config, rsslib

# --- Useful classes

class Story:

    def __init__(self, site, item):
        self._site = site
        self._item = item
        self._time = time.time()

    def get_title(self):
        return self._item.get_title()

    def get_site_title(self):
        return self._site.get_title()

    def get_link(self):
        return self._item.get_link()

    def get_description(self):
        return self._item.get_description()

    def get_item(self):
        return self._item

    def get_characteristic(self):
        return self.get_title() + " " + self.get_link()

    def get_time(self):
        return self._time
    
    def __cmp__(self, other):
        if isinstance(other, Story):
            res = cmp(self.get_site_title(), other.get_site_title())
            if res == 0:
                res = cmp(self.get_time(), other.get_time())
            if res == 0:
                res = cmp(self.get_title(), other.get_title())
            return res
        else:
            return -1


class StoryCache:

    def __init__(self, file):
        try:
            inf = pickle.Unpickler(open(file, "rb"))
            self._read = inf.load()
            self._unread = inf.load()

            self._expire(self._read)
            self._expire(self._unread)
        except IOError:
            self._read = {}
            self._unread = {}

    def save(self, file):
        pickler = pickle.Pickler(open(file, "wb"))
        pickler.dump(self._read)
        pickler.dump(self._unread)
            
    def get_old_unread(self):
        return self._unread.values()
    
    def show(self, story):
        if self._read.has_key(story.get_characteristic()) or \
           self._unread.has_key(story.get_characteristic()):
            return 0
        else:
            self._unread[story.get_characteristic()] = story
            return 1

    def read(self, story):
        del self._unread[story.get_characteristic()]
        self._read[story.get_characteristic()] = story

    def _expire(self, stories):
        now = time.time()
        for (key, story) in stories.items():
            if story.get_time() + config.get_expiry_wait() < now:
                del stories[key]
        

# --- ErrorRecorder

class ErrorRecorder:

    def __init__(self):
        self._errors = []
    
    def error(self, exception):
        self._errors.append(str(exception))

    def fatalError(self, exception):
        self._errors.append(str(exception))

    def warning(self, exception):
        self._errors.append(str(exception))

        
# --- Function interface
    
def save():
    cache.save(config.cache_file_location())

def get_summaries(error_handler):
    summaries = []
    
    for source in config.get_sources():
        try:
            summary = rsslib.load_rss(source.get_url())
            source.set_title(summary.get_title())
            source.set_descr(summary.get_description())
            summaries.append(summary)
        except Exception, e:
            error_handler.fatalError(SAXException(str(e), e))

    return summaries

def get_sources():
    return config.get_sources()

# --- Global variables

cache = StoryCache(config.cache_file_location())

