"""Functionality for finding important files and user configuration.

$Id: config.py,v 1.3 2001/01/12 08:16:54 larsga Exp $
"""

from socket import gethostbyaddr, gethostname
import os, string
import rsslib

# ===== File location logic

def code_location():
    path = os.path.split(rsslib.__file__)[0]
    if path == "": path = "."
    return path + os.sep

def cache_file_location():
    if os.environ.has_key("HOME"):
        return os.environ["HOME"] + os.sep + ".rss-cache.mar"
    else: 
        return code_location() + "rss-cache.mar"
            
def config_file_location():
    if os.environ.has_key("HOME"):
        return os.environ["HOME"] + os.sep + ".rssconfig.xml"
    else:
        return code_location() + "rss-config.xml"

def bitmap_file_location():
    return code_location() + "icons" + os.sep


# ===== User configuration

# --- Global variables

sources = []
use_threading = 0
show_errors = 1
expiry_wait = 28 # days
list_size = 80 # percent of full window

mail_server = "localhost"
recipients = ["%s@%s" % (os.environ["USER"],
                         gethostbyaddr(gethostname())[0])]
from_addr = "%s@%s" % (os.environ["USER"],
                       gethostbyaddr(gethostname())[0])


# --- Useful classes

class Source:
    "Represents an RSS channel."

    def __init__(self, url, title = "", descr = ""):
        self._url = url
        self._title = title
        self._descr = descr
    
    def get_title(self):
        return self._title
    
    def get_url(self):
        return self._url
    
    def get_descr(self):
        return self._descr        

    def set_title(self, title):
        self._title = title

    def set_descr(self, descr):
        self._descr = descr

        
# --- Function interface

def save():
    outf = open(config_file_location(), "w")
    outf.write('<rssconfig version="1.0">\n')

    for source in sources:
        outf.write('  <source url="%s"' % source.get_url())
        if source.get_title():
            outf.write(' title="%s"' % source.get_title())
        if source.get_descr():
            outf.write(' descr="%s"' % source.get_descr())
        outf.write('/>\n')


    outf.write(('  <clientconfig show-errors="%s" '
                'use-threading="%s" expiry-wait-days="%s" '
                'list-size="%s"/>\n\n') %
               (get_show_errors(), get_use_threading(), expiry_wait,
                get_list_size()))

    outf.write(('  <emailconfig mail-server="%s" recipients="%s" '
                'from-addr="%s"/>\n\n') %
               (get_mail_server(), string.join(get_recipients(), " "),
                get_from_addr()))
    
    outf.write('</rssconfig>\n')
    outf.close()

    
def get_sources():
    return sources

def get_show_errors():
    return show_errors

def get_use_threading():
    return use_threading

def get_expiry_wait():
    return expiry_wait * 86400

def get_list_size():
    return list_size

def set_list_size(size):
    global list_size
    list_size = size

def get_mail_server():
    return mail_server

def get_recipients():
    return recipients

def get_from_addr():
    return from_addr
    

# --- Internal

from xml.sax.handler import ContentHandler

class ConfigHandler(ContentHandler):

    def startElement(self, name, attrs):
        global show_errors, use_threading, expiry_wait, list_size
        global recipients, from_addr, mail_server

        if name == "source":
            sources.append(Source(attrs["url"],
                                  attrs.get("title"),
                                  attrs.get("descr")))

        elif name == "clientconfig":
            show_errors = attrs.get("show-errors", 1)
            use_threading = attrs.get("use-threading", 0)
            expiry_wait = attrs.get("expiry-wait", 28)
            list_size = int(attrs.get("list-size", 80))

        elif name == "emailconfig":
            mail_server = attrs.get("mail-server", mail_server)
            recipients  = string.split(attrs.get("recipients",
                                  string.join(recipients, " ")))
            from_addr   = attrs.get("from-addr", from_addr)
            

def _load():
    from xml.sax import parse
    try:
        parse(config_file_location(), ConfigHandler())
    except IOError, e:
        if e.errno == 2: 
            pass  # no config file, that is, use defaults
        else:
            raise # some other error; notify user

_load()


del _load, ContentHandler, ConfigHandler

