"""A wxPython-based RSS editor.

$Id: rss-editor.py,v 1.3 2001/01/12 08:16:54 larsga Exp $
"""

from wxPython.wx import *
from wxUtils import *
from xml.sax._exceptions import SAXException, SAXParseException
import rsslib, sys

on_windows = sys.platform == "win32"
version    = "0.01"
        
# --- Application

class RSSEditor(wxApp):

    def __init__(self, arg, file = None):
        self.file = file 
        wxApp.__init__(self, arg)
    
    def OnInit(self):
        frame = EditorWindow(NULL, -1, "RSS-Editor " + str(version))
        frame.Show(true)
        self.SetTopWindow(frame)

        if self.file != None:
            frame.open_file(self.file)
        
        return true

    
# --- Editor window

ID_NEW     = NewId()
ID_OPEN    = NewId()
ID_SAVE    = NewId()
ID_SAVE_AS = NewId()
ID_ABOUT   = NewId()
ID_EXIT    = NewId()

ID_NEW_ITEM = NewId()
ID_DEL_ITEM = NewId()
ID_UP_ITEM  = NewId()
ID_DN_ITEM  = NewId()

ID_ITEM_LIST = NewId()

class EditorWindow(wxFrame):
    def __init__(self, parent, ID, title):
        wxFrame.__init__(self, parent, ID, title,
                         wxDefaultPosition, wxSize(550, 450))

        self.CreateStatusBar()
        self.SetAutoLayout(true)
        
        # --- Menu
        
        menu = wxMenu()
        menu.Append(ID_NEW,  "&New file", 
                    "Open a file for editing")
        menu.Append(ID_OPEN,  "&Open file...", 
                    "Open a file for editing")
        menu.Append(ID_SAVE,  "&Save",
                    "Save the current file")
        menu.Append(ID_SAVE_AS,  "Save &as...",
                    "Save the current file in a specified location")
        menu.AppendSeparator()
        menu.Append(ID_ABOUT, "&About",
                    "Information about the editor")
        menu.Append(ID_EXIT, "E&xit", "Terminate the editor")

        menuBar = wxMenuBar()
        menuBar.Append(menu, "&File");

        self.SetMenuBar(menuBar)

        EVT_MENU(self, ID_NEW,     self.NewDocument)
        EVT_MENU(self, ID_OPEN,    self.Open)
        EVT_MENU(self, ID_SAVE,    self.Save)
        EVT_MENU(self, ID_SAVE_AS, self.SaveAs)
        EVT_MENU(self, ID_ABOUT,   self.OnAbout)
        EVT_MENU(self, ID_EXIT,    self.TimeToQuit)
        
        # --- File controls

        # channel controls
        channel_book = wxNotebook(self,-1)
        item_panel = wxPanel(self,-1)
        lc = wxLayoutConstraints()
        lc.top.SameAs(self, wxTop)
        lc.height.Absolute(125
)
        lc.left.SameAs(self, wxLeft)
        lc.right.SameAs(self, wxRight)
        channel_book.SetConstraints(lc)

        
        # main
        main_panel = wxPanel(channel_book, -1)
        main_panel.SetAutoLayout(true)
        putter = RowPairPutter(main_panel)
        
        self.title = putter.add_pair("Title: ",
                             wxTextCtrl(putter.get_parent(), -1, ""))
        self.link  = putter.add_pair("Link: ",                           
                             wxTextCtrl(putter.get_parent(), -1, ""))
        self.descr = putter.add_pair("Description: ",                    
                             wxTextCtrl(putter.get_parent(), -1, ""))

        channel_book.AddPage(main_panel, "Main")

        # contacts
        contacts_panel = wxPanel(channel_book, -1)
        contacts_panel.SetAutoLayout(true)
        putter = RowPairPutter(contacts_panel)
        
        self.editor = putter.add_pair("Editor: ",
                               wxTextCtrl(putter.get_parent(),-1,""))
        self.webmaster = putter.add_pair("Webmaster: ",
                               wxTextCtrl(putter.get_parent(),-1,""))

        channel_book.AddPage(contacts_panel, "Contacts")

        # metadata
        meta_panel = wxPanel(channel_book, -1)
        meta_panel.SetAutoLayout(true)
        putter = RowPairPutter(meta_panel)
        
        self.lang = putter.add_pair("Language: ",
                               wxTextCtrl(putter.get_parent(),-1,""))
        self.rating = putter.add_pair("Rating: ",
                               wxTextCtrl(putter.get_parent(),-1,""))
        self.copyright = putter.add_pair("Copyright: ",
                               wxTextCtrl(putter.get_parent(),-1,""))

        channel_book.AddPage(meta_panel, "Metadata")

        
        
        # item controls
        # item_panel created higher up
        item_panel.SetAutoLayout(true)
        lc = wxLayoutConstraints()
        lc.top.SameAs(channel_book, wxBottom)
        lc.bottom.SameAs(self, wxBottom)
        lc.left.SameAs(self, wxLeft)
        lc.right.SameAs(self, wxRight)
        item_panel.SetConstraints(lc)

        items = wxListCtrl(item_panel, ID_ITEM_LIST,
                           wxDefaultPosition, wxDefaultSize,
                           wxLC_REPORT | wxSUNKEN_BORDER)
        items.InsertColumn(0, "Title")
        text_panel = wxPanel(item_panel, -1)
        lc = wxLayoutConstraints()
        lc.top.SameAs(item_panel, wxTop)
        lc.bottom.SameAs(text_panel, wxTop)
        lc.left.SameAs(item_panel, wxLeft)
        lc.right.SameAs(item_panel, wxRight)
        items.SetConstraints(lc)

        # text_panel created above
        button_panel = wxPanel(item_panel, -1)
        text_panel.SetAutoLayout(true)
        lc=wxLayoutConstraints()
        lc.height.Absolute(70)
        lc.bottom.SameAs(item_panel, wxBottom)
        lc.left.SameAs(item_panel, wxLeft)
        lc.right.SameAs(button_panel, wxLeft)
        text_panel.SetConstraints(lc)

        putter = RowPairPutter(text_panel)
        title = putter.add_pair("Title: ",
                              wxTextCtrl(putter.get_parent(),-1,""))
        link  = putter.add_pair("Link: ",
                              wxTextCtrl(putter.get_parent(),-1,""))
        descr  = putter.add_pair("Description: ",
                              wxTextCtrl(putter.get_parent(), -1, "",
                                     wxDefaultPosition, wxDefaultSize,
                                     wxTE_MULTILINE))

        # button_panel created above
        button_panel.SetAutoLayout(true)
        lc=wxLayoutConstraints()
        lc.height.SameAs(text_panel, wxHeight)
        lc.bottom.SameAs(item_panel, wxBottom)
        lc.width.Absolute(70)
        lc.right.SameAs(item_panel, wxRight)
        button_panel.SetConstraints(lc)

        putter = RowPutter(button_panel)
        new = putter.put(wxButton(button_panel, ID_NEW_ITEM, "New"))
        delete = putter.put(wxButton(button_panel, ID_DEL_ITEM,
                                     "Delete"))
        up = putter.put(wxButton(button_panel, ID_UP_ITEM, "Up"))
        down = putter.put(wxButton(button_panel, ID_DN_ITEM, "Down"))

        EVT_BUTTON(new, ID_NEW_ITEM, self.NewItem)
        EVT_BUTTON(delete, ID_DEL_ITEM, self.DelItem)
        EVT_BUTTON(up, ID_UP_ITEM, self.UpItem)
        EVT_BUTTON(down, ID_DN_ITEM, self.DownItem)
        
        # --- Internal data
        self.file = None

        self.ss = rsslib.SiteSummary(None)
        self.titlec = TextMethodConnector(self.ss, "title",
                                          self.title)
        self.linkc = TextMethodConnector(self.ss, "link",
                                         self.link)
        self.descrc = TextMethodConnector(self.ss, "description",
                                          self.descr)
        self.editorc = TextMethodConnector(self.ss, "managing_editor",
                                           self.editor)
        self.masterc = TextMethodConnector(self.ss, "web_master",
                                           self.webmaster)
        self.langc = TextMethodConnector(self.ss, "language",
                                         self.lang)
        self.ratingc = TextMethodConnector(self.ss, "rating",
                                           self.rating)
        self.copyrightc = TextMethodConnector(self.ss, "copyright",
                                             self.copyright)
        
        
        # pub_date
        # last_build_date
        # docs
        # skip_days
        # skip_hours
        
        item = rsslib.Item("", "")
        self.itemc = ListConnector(self.ss.get_items(), items,
             lambda c,ix,it: c.InsertStringItem(ix,str(it.get_title())))
        self.itemc.subconnectors = [
          TextMethodConnector(item, "title", title, self.itemc),
          TextMethodConnector(item, "link", link),
          TextMethodConnector(item, "description", descr)]
        
    # --- Actions
        
    def NewDocument(self, event):
        self.connect(rsslib.SiteSummary(None))

    def NewItem(self, event):
        self.itemc.add(0, rsslib.Item("[New]", "", ""))

    def DelItem(self, event):
        self.itemc.delete_selected()

    def UpItem(self, event):
        ix = self.itemc.tracker.current_ix
        if ix == None or ix < 1:
            return

        self.itemc.exchange(ix,ix-1)
        self.itemc.select(ix-1)
        self.itemc.unselect(ix)

    def DownItem(self, event):
        ix = self.itemc.tracker.current_ix
        if ix == None or ix+1 >= len(self.ss.items):
            return

        self.itemc.exchange(ix,ix+1)
        self.itemc.select(ix+1)
        self.itemc.unselect(ix)
                
    def Open(self, event):
        if on_windows:
            wildcards = "RSS documents (*.rss)|*.rss|"\
                        "RSS documents (*.rdf)|" \
                        "*.rdf|RSS documents (*.xml)|*.xml|" \
                        "All files (*.*)" \
                        "|*.*"
        else:
            wildcards = "*.rss"
        
        file = wxFileSelector("Choose a file to open", "", "", "",
                              wildcards, wxOPEN | wxHIDE_READONLY)
        if file == "": return
        self.open_file(file)
        
    def SaveAs(self, event):
        if self.ss == None: return
        file = wxFileSelector("Choose a file to save in", "", "",
                              "", "", wxSAVE | wxOVERWRITE_PROMPT)
        if file == "": return
        self.file = file
        self.Save(None)

    def Save(self, event):
        if self.ss == None: return
        file = self.file or wxFileSelector("Choose a file to save in",
                                           "","","","",
                                           wxSAVE | wxOVERWRITE_PROMPT)
        if file == "": return
        
        rsslib.write_rss_091(self.ss, self.file)
        self.file = file
        
    def OnAbout(self, event):
        dlg = wxMessageDialog(self, "A simple RSS file editor.\n\n" +
                         "Version: " + version + "\n" +
                         "rsslib version: " + rsslib.version + "\n",
                         "About RSS-Editor", wxOK | wxICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()

    def TimeToQuit(self, event):
        self.Close(true)

    # --- Internal functions

    def connect(self, ss):
        self.ss = ss
        self.titlec.set_object(self.ss)
        self.linkc.set_object(self.ss)
        self.descrc.set_object(self.ss)
        self.editorc.set_object(self.ss)
        self.masterc.set_object(self.ss)
        self.langc.set_object(self.ss)
        self.ratingc.set_object(self.ss)
        self.copyrightc.set_object(self.ss)

        self.itemc.set_list(self.ss.get_items())

    def open_file(self, file):
        try:
            self.connect(rsslib.load_rss(file))
            self.file = file
        except SAXParseException, e:
            wxMessageBox("Parse error: " + e.getMessage() +
                         " on %s:%s" % (e.getLineNumber(),
                                        e.getColumnNumber()), "Error")
        except Exception, e:
            wxMessageBox(str(e), "Error")
        
# --- Main program

if len(sys.argv) > 1:
    file = sys.argv[1]
else:
    file = None
    
app = RSSEditor(0, file)
app.MainLoop()
