# This parser filter joins consecutive characters events together into
# single events.

from xml.sax.saxutils import XMLFilterBase

feature_split_on_pis = "http://garshol.priv.no/symbolic/sax/cjf/split-on-pis"

class CharactersJoinFilter(XMLFilterBase):

    # configuration
    
    def __init__(self, parent = None):
        XMLFilterBase.__init__(self, parent)
        self._contents = ""
        self._split_on_pis = 0
        
    def set_split_on_pis(self, split):
        """Controls whether the filter allows PI occurrences to split
        characters events."""
        self._split_on_pis = split


    # features / properties

    def getFeature(self, name):
        if name == feature_split_on_pis:
            return self._split_on_pis
        return self._parent.getFeature(name)

    def setFeature(self, name, state):
        if name == feature_split_on_pis:
            self._split_on_pis = state
        self._parent.setFeature(name, state)


    # events
    
    def characters(self, content):
        self._contents = self._contents + content

    def startElement(self, name, attrs):
        if self._contents:
            self._cont_handler.characters(self._contents)
            self._contents = ""
        self._cont_handler.startElement(name, attrs)
    
    def endElement(self, name):
        if self._contents:
            self._cont_handler.characters(self._contents)
            self._contents = ""
        self._cont_handler.endElement(name)
    

    def processingInstruction(self, target, data):
        if self._split_on_pis and self._contents:
            self._cont_handler.characters(self._contents)
            self._contents = ""
        self._cont_handler.processingInstruction(target, data)

