
import sys, string
from saxtracker import SAXTracker
from xml.saxtools.drivers import drv_pysp

# --- Utilities

def normalize(str):
    return string.join(string.split(str), " ")

# --- The ContentHandler

class ToCHandler(SAXTracker):

    def __init__(self, out = None):
        SAXTracker.__init__(self, ["TITLE", "H1", "H2", "H3", "H4"])
        self._out = out or sys.stdout
        self._numbers = [0]

    def endElement(self, name):
        SAXTracker.endElement(self, name)

        level = 0
        if name == "TITLE" and self._elemstack[-1] in \
           ("CHAPOPEN", "APPENDIX"):
            self._numbers = [self._numbers[0] + 1]
            self._out.write("\n%d. %s\n" % (self._numbers[0],
                                            normalize(self._contents)))

        elif name == "H1":
            level = 1
        elif name == "H2":
            level = 2
        elif name == "H3":
            level = 3

        if level:
            if len(self._numbers) == level:
                self._numbers = self._numbers[ : level] + [1]
            else:
                self._numbers = self._numbers[ : level] + \
                                [self._numbers[level] + 1]

            self._out.write(("  " * level))
            for number in self._numbers[ : level + 1]:
                self._out.write("%d." % number)
                
            self._out.write(" %s\n" % normalize(self._contents))
            
# --- Main program

if __name__ == "__main__":
    parser = drv_pysp.create_parser()
    parser.setProperty(drv_pysp.property_catalogs, [sys.argv[2]])
    parser.setContentHandler(ToCHandler())
    parser.parse(sys.argv[1])
