import rfc822
from xml.sax.xmlreader import XMLReader, AttributesImpl
from xml.sax.handler import feature_namespaces
from xml.sax.saxutils import prepare_input_source, XMLGenerator
from xmlgen import XMLWriter, XMLWriterNS

try:
    import codecs
    use_unicode = 1
except ImportError:
    use_unicode = 0 # assume Python 1.5.2

# --- Constants

header_elements = {"from"    : "from",
                   "subject" : "subject",
                   "to"      : "to"      }

mail_namespace = "http://garshol.priv.no/symbolic/sax/mail/0.1"


# --- The XMLReader

class MailParser(XMLReader):

    def __init__(self):
        XMLReader.__init__(self)
        self._namespaces = 0

    def getFeature(self, name):
        if name == feature_namespaces:
            return self._namespaces
        
        raise SAXNotRecognizedException("Feature '%s' not recognized"
                                        % name)
        
    def setFeature(self, name, state):
        if name == feature_namespaces:
            self._namespaces = state
            return
        
        raise SAXNotRecognizedException("Feature '%s' not recognized"
                                        % name)


    def parse(self, source):
        if self._namespaces:
            writer = XMLWriterNS(self._cont_handler, mail_namespace)
        else:
            writer = XMLWriter(self._cont_handler)
            
        
        source = prepare_input_source(source)
        if use_unicode:
            (enc, dec, Reader, Writer) = codecs.lookup("iso-8859-1")
            stream = Reader(source.getByteStream())
        else:
            stream = source.getByteStream()


        writer.startDocument()
        writer.startElement("email")
        writer.startElement("headers")
        
        message = rfc822.Message(stream)
        for header in message.keys():
            if header_elements.has_key(header):
                elem = header_elements[header]
                attrs = {}
            else:
                elem = "header"
                attrs = {"name" : header}

            writer.fieldElement(elem, message[header], attrs)
            writer.characters("\n")

        writer.endElement("headers")


        message.rewindbody()

        body = stream.read()
        print type(body)
        writer.fieldElement("body", body)

        writer.endElement("email")
        writer.endDocument()


if __name__ == "__main__":
    import sys
    p = MailParser()
    p.setContentHandler(XMLGenerator())
    p.parse(sys.argv[1])
