
import sys
from xml.sax import parse
from saxtracker import SAXTracker

# --- Templates

top = \
"""
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<HTML>
<HEAD>
  <TITLE>%s</TITLE>
  <LINK REL=stylesheet HREF="play.css" TYPE="text/css">
</HEAD>

<BODY>

<H1>%s</H1>
"""

bottom = \
"""
<HR>

<ADDRESS>
Converted from Jon Bosak's XML to HTML with SAX.
</ADDRESS>

</BODY>
</HTML>
"""

# --- The ContentHandler

class PlayToHtmlHandler(SAXTracker):

    def __init__(self, out = None):
        SAXTracker.__init__(self, ["TITLE", "P", "PERSONA", "GRPDESCR",
                                   "SPEAKER", "LINE", "STAGEDIR"])
        self._out = out or sys.stdout
        
        self._start_dispatch = {"FM"       : self.start_FM,
                                "PERSONAE" : self.start_PERSONAE,
                                "PGROUP"   : self.start_PGROUP,
                                "PROLOGUE" : self.start_PROLOGUE,
                                "EPILOGUE" : self.start_EPILOGUE,
                                "LINE"     : self.noop,
                                "SPEECH"   : self.noop,
                                "SPEAKER"  : self.noop,
                                "STAGEDIR" : self.noop }
        
        self._end_dispatch = {"LINE"     : self.end_LINE,
                              "SPEAKER"  : self.end_SPEAKER,
                              "STAGEDIR" : self.end_STAGEDIR,
                              "TITLE"    : self.end_TITLE,
                              "P"        : self.end_P,
                              "PERSONA"  : self.end_PERSONA,
                              "GRPDESCR" : self.end_GRPDESCR,
                              "PERSONAE" : self.end_PERSONAE}

    def startElement(self, name, attrs):
        try:
            self._start_dispatch[name] ()
        except KeyError:
            pass

        SAXTracker.startElement(self, name, attrs)

    def endElement(self, name):
        SAXTracker.endElement(self, name)

        try:
            self._end_dispatch[name] ()
        except KeyError:
            pass

        self._contents = ""

    def endDocument(self):
        self._out.write(bottom)
        
    # --- start element handlers
    
    # front matter

    def start_FM(self):
        self._out.write("<h2>Front matter</h2>\n")

    def start_PERSONAE(self):
        self._out.write("<h2>Dramatis personae</h2>\n")
        self._out.write("<ul>")

    def start_PGROUP(self):
        self._out.write("<li>")

    def start_PROLOGUE(self):
        self._out.write("<h2>Prologue</h2>\n")

    def start_EPILOGUE(self):
        self._out.write("<h2>Epilogue</h2>\n")

    # --- end element handlers
        
    def end_LINE(self):
        self._out.write("<p class=line>%s</p>" % self._contents)

    def end_SPEAKER(self):
        if self._elemstack[-2] != "EPILOGUE" and \
           self._elemstack[-2] != "PROLOGUE":
            self._out.write("<p class=speaker>%s:</p>\n" %
                            self._contents)
            
    def end_STAGEDIR(self):
        self._out.write("<p class=stagedir>%s</p>" % self._contents)
            
    def end_TITLE(self):
        if self._elemstack[-1] == "SCENE":
            self._out.write("<h3>%s</h3>" % self._contents)
                
        elif self._elemstack[-1] == "ACT" or \
             self._elemstack[-1] == "INDUCT":
            self._out.write("<h2>%s</h2>" % self._contents)

        elif self._elemstack[-1] == "PLAY":
            self._out.write(top % (self._contents, self._contents))

    def end_P(self):
        self._out.write("<p>%s</p>\n" % self._contents)

    def end_PERSONA(self):
        if self._elemstack[-1] == "PERSONAE":
            self._out.write("<li>%s</li>\n" % self._contents)
        else: # group
            self._out.write(self._contents + ", ")

    def end_GRPDESCR(self):
        self._out.write(self._contents + "</li>\n")
                
    def end_PERSONAE(self):
        self._out.write("</ul>\n")

    import os
    noop = os.getcwd
        
# --- Conversion function

def convert(infile, out):
    outf = open(out, "w")

    parse(infile, PlayToHtmlHandler(outf))
    
    outf.close()
        
# --- Main program

if __name__ == "__main__":
    convert(sys.argv[1], "out.html")
