import sys
from saxtracker import SAXTracker
from xml.sax import handler, parse

# --- Templates

top = \
"""
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<HTML>
<HEAD>
  <TITLE>%s</TITLE>
  <LINK REL=stylesheet HREF="play.css" TYPE="text/css">
</HEAD>

<BODY>

<H1>%s</H1>
"""

bottom = \
"""
<HR>

<ADDRESS>
Converted from Jon Bosak's XML to HTML with SAX.
</ADDRESS>

</BODY>
</HTML>
"""


# --- The ContentHandler

class PlayToHtmlHandler(SAXTracker):

    def __init__(self, out = None):
        SAXTracker.__init__(self, ["TITLE", "P", "PERSONA", "GRPDESCR",
                                   "SPEAKER", "LINE", "STAGEDIR"])
        self._out = out or sys.stdout


    def startElement(self, name, attrs):
        # front matter
        
        if name == "FM": 
            self._out.write("<h2>Front matter</h2>\n")

        elif name == "PERSONAE":
            self._out.write("<h2>Dramatis personae</h2>\n")
            self._out.write("<ul>")

        elif name == "PGROUP":
            self._out.write("<li>")

        # play

        elif name == "PROLOGUE":
            self._out.write("<h2>Prologue</h2>\n")
        
        elif name == "EPILOGUE":
            self._out.write("<h2>Epilogue</h2>\n")
        
        SAXTracker.startElement(self, name, attrs)


    def endElement(self, name):
        SAXTracker.endElement(self, name)

        # front matter
        
        if name == "TITLE" and self._elemstack[-1] == "PLAY":
            self._out.write(top % (self._contents, self._contents))

        elif name == "P":
            self._out.write("<p>%s</p>\n" % self._contents)


        elif name == "PERSONA":
            if self._elemstack[-1] == "PERSONAE":
                self._out.write("<li>%s</li>\n" % self._contents)
            else: # group
                self._out.write(self._contents + ", ")

        elif name == "GRPDESCR":
            self._out.write(self._contents + "</li>\n")
                
        elif name == "PERSONAE":
            self._out.write("</ul>\n")


        # play

        elif name == "TITLE" and \
             (self._elemstack[-1] in ("ACT", "INDUCT")):
            self._out.write("<h2>%s</h2>" % self._contents)
        
        elif name == "TITLE" and self._elemstack[-1] == "SCENE":
            self._out.write("<h3>%s</h3>" % self._contents)

        elif name == "SPEAKER" and \
             self._elemstack[-2] != "EPILOGUE" and \
             self._elemstack[-2] != "PROLOGUE":
            self._out.write("<p class=speaker>%s:</p>\n" % self._contents)

        elif name == "LINE":
            self._out.write("<p class=line>%s</p>" % self._contents)

        elif name == "STAGEDIR":
            self._out.write("<p class=stagedir>%s</p>" %
                            self._contents)

        self._contents = ""


    def endDocument(self):
        self._out.write(bottom)


# --- Conversion function

def convert(infile, out):
    if sys.hexversion > 0x2000000:
        import codecs
        outf = codecs.open(out, "w", "iso-8859-1")
    else:
        outf = open(out, "w")
        
    parse(infile, PlayToHtmlHandler(outf))
    outf.close()

    
# --- Main program

if __name__ == "__main__":
    convert(sys.argv[1] , sys.argv[2])
