import sys
from xml.sax import handler, make_parser

# --- Templates

top = \
"""
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<HTML>
<HEAD>
  <TITLE>%s</TITLE>
</HEAD>

<BODY>
<A HREF="%s"><H1>%s</H1></A>
"""

bottom = \
"""       
<HR>

<ADDRESS>
Converted to HTML using SAX 2.0.
</ADDRESS>

</BODY>
</HTML>
"""

# --- ContentHandler

rssns = "http://purl.org/rss/1.0/"

class RSS2HTMLHandler(handler.ContentHandler):

    def __init__(self, out = None):
        self._out = out or sys.stdout

        # tracking state
        self._data = ""
        self._first_item = 1
        self._top_done = 0

        self._title = None
        self._link = None
        self._descr = None
        
    def startElementNS(self, name, qname, attrs):
        self._data = ""

        if (not self._top_done) and \
           (name == (rssns, "item") or name == (rssns, "image")):
            self._out.write(top %
                            (self._title, self._link, self._title))

            if self._descr != None:
                self._out.write("<p>%s</p>" % self._descr)

            self._top_done = 1
            
        elif name == (rssns, "item"):
            self._descr = None # reset for this item
            if self._first_item:                
                self._out.write("\n<ul>\n")

            self._first_item = 0
            
    def endElementNS(self, name, qname):
        if name == (rssns, "title"):
            self._title = self._data

        elif name == (rssns, "link"):
            self._link = self._data

        elif name == (rssns, "description"):
            self._descr = self._data

        elif name == (rssns, "item"):
            self._out.write('  <li><a href="%s">%s</a> %s\n' %
                       (self._link, self._title, self._descr or ""))

    def endDocument(self):
        self._out.write("</ul>\n")
        self._out.write(bottom)
            
    def characters(self, chars):
        self._data = self._data + chars

# --- Conversion function

def htmlconv(infile, outfile):
    outf = open(outfile, "w")
    parser = make_parser()
    parser.setContentHandler(RSS2HTMLHandler(outf))
    parser.setFeature(handler.feature_namespaces, 1)
    parser.parse(infile)
    outf.close()

# --- Main program
    
if __name__ == "__main__":
    import sys
    htmlconv(sys.argv[1], sys.argv[2])
