import sys, codecs
from xml.sax import handler, parse

# --- Templates

top = \
"""
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<HTML>
<HEAD>
  <TITLE>%s</TITLE>
</HEAD>

<BODY>
<A HREF="%s"><H1>%s</H1></A>
"""

bottom = \
"""       
<HR>

<ADDRESS>
Converted to HTML using SAX 2.0.
</ADDRESS>

</BODY>
</HTML>
"""

# --- ContentHandler

class RSS2HTMLHandler(handler.ContentHandler):

    def __init__(self, out = None):
        self._out = out or sys.stdout

        # tracking state
        self._data = ""
        self._first_item = 1
        self._top_done = 0

        self._title = None
        self._link = None
        self._descr = None
        
    def startElement(self, name, attrs):
        self._data = ""

        if (not self._top_done) and (name == "item" or name == "image"):
            self._out.write(top % (self._title, self._link, self._title))

            if self._descr != None:
                self._out.write("<p>%s</p>" % self._descr)

            self._top_done = 1
        elif name == "item":
            self._descr = None # reset for this item
            if self._first_item:                
                self._out.write("\n<ul>\n")

            self._first_item = 0
            
    def endElement(self, name):
        if name == "title":
            self._title = self._data

        elif name == "link":
            self._link = self._data

        elif name == "description":
            self._descr = self._data

        elif name == "item":
            self._out.write('  <li><a href="%s">%s</a> %s\n' %
                            (self._link, self._title, self._descr or ""))

    def endDocument(self):
        self._out.write("</ul>\n")
        self._out.write(bottom)
            
    def characters(self, chars):
        self._data = self._data + chars

# --- Conversion function

def htmlconv(infile, out):
    outf = codecs.open(out, "w", "iso-8859-1")
    parse(infile, RSS2HTMLHandler(outf))
    outf.close()

# --- Main program
    
if __name__ == "__main__":
    import sys
    htmlconv(sys.argv[1], sys.argv[2])
