import sys, codecs
from saxtracker2 import SAXTracker
from xml.sax import parse

# --- Templates

top = \
"""
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<HTML>
<HEAD>
  <TITLE>%s</TITLE>
</HEAD>

<BODY>
<A HREF="%s"><H1>%s</H1></A>
"""

bottom = \
"""       
<HR>

<ADDRESS>
Converted to HTML using SAX 2.0.
</ADDRESS>

</BODY>
</HTML>
"""


# --- ContentHandler

class RSS2HTMLHandler(SAXTracker):

    def __init__(self, out = None):
        SAXTracker.__init__(self, {"title" : "_title",
                                   "link" : "_link",
                                   "description" : "_descr"})
        self._out = out or sys.stdout

        # tracking state
        self._first_item = 1


    def startElement(self, name, attrs):
        if name == "item":
            self._descr = None # reset for this item
            if self._first_item:
                self._out.write("\n<ul>\n")

            self._first_item = 0

        SAXTracker.startElement(self, name, attrs)


    def endElement(self, name):
        SAXTracker.endElement(self, name)

        if name == "channel":
            self._out.write(top %
                            (self._title, self._link, self._title))

            if self._descr != None:
                self._out.write("<p>%s</p>" % self._descr)

        elif name == "item":
            self._out.write('  <li><a href="%s">%s</a> %s\n' %
                      (self._link, self._title, self._descr or ""))


    def endDocument(self):
        self._out.write("</ul>\n")
        self._out.write(bottom)

# --- Conversion function

def htmlconv(infile, out):
    outf = codecs.open(out, "w", "iso-8859-1")
    parse(infile, RSS2HTMLHandler(outf))
    outf.close()
