# This application processes documents to gather statistical
# information about them.

import sys
import operator

from xml.sax import parse, handler
        
# ====== SAX APPLICATION
        
# --- ContentHandler

class StatsHandler(handler.ContentHandler):

    def __init__(self):
        self._elems       = {}
        self._attrs       = {}
        self._attrlengths = {}
        self._pis         = {}
        self._pilengths   = {}
        
    def startElement(self, name, attrs):
        self._elems[name] = self._elems.get(name, 0) + 1
        for (aname, value) in attrs.items():
            self._attrs[(name, aname)] = \
                    self._attrs.get((name, aname), 0) + 1
            self._attrlengths[(name, aname)] = \
                    self._attrlengths.get((name, aname), 0) + len(value)

    def processingInstruction(self, target, data):
        self._pis[target] = self._pis.get(target, 0) + 1
        self._pilengths[target] = \
                    self._pilengths.get(target, 0) + len(data)

    def report(self):
        print "===== DOCUMENT REPORT"
        print
        print "---ELEMENT TYPES"
        print "%20s     %5s" % ("Type", "Count")
        for (name, count) in self._elems.items():
            print "%20s     %5s" % (name, count)
        print

        print "---ATTRIBUTES"
        attrs = self._attrs.keys()
        attrs.sort()
        format = "%20s     %20s     %5s     %5s"
        print format % ("Element","Attribute","Occur","Avgln")
        for (ename, aname) in attrs:
            count = self._attrs[(ename, aname)]
            print format % (ename, aname, count,
                            self._attrlengths[(ename, aname)] / count)
        print
        
        print "---PI TARGETS"
        print "%20s     %5s" % ("Type", "Count")
        for (name, count) in self._pis.items():
            print "%20s     %5s" % (name, count)
        print


# --- ErrorHandler

class CountErrors:

    def __init__(self):
        self._warnings = 0
        self._errors   = 0
        self._fatals   = 0

    def error(self, exception):
        self._errors = self._errors + 1

    def fatalError(self, exception):
        self._fatals = self._fatals + 1

    def warning(self, exception):
        self._warnings = self._warnings + 1

    def report(self):
        print "---ERRORS"
        print "Warnings    ",self._warnings
        print "Errors      ",self._errors
        print "Fatal errors",self._fatals
        print

	        
# --- Main program

stats = StatsHandler()
errors = CountErrors()

p = parse(sys.argv[1], stats, errors)

stats.report()
errors.report()
