# This script converts XBEL bookmark collections to web pages, using
# application-specific state tracking.

import codecs
from xml.sax import handler, parse

# --- Templates

top = \
"""
<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">
<HTML>
<HEAD>
  <TITLE>%s</TITLE>
</HEAD>

<BODY>
<H1>%s</H1>
"""

bottom = \
"""        
<HR>

<ADDRESS>
Converted by xbel2html.py, using SAX.
</ADDRESS>

</BODY>
</HTML>
"""


# --- ContentHandler

class XBELHandler(handler.ContentHandler):

    ROOT        = 0
    FOLDER      = 1
    BOOKMARK    = 2
    NONE        = 3
    
    def __init__(self, out):
        self._out = out
        self._inside = XBELHandler.NONE # what context are we in?
        self._contents = ""
        self._url = None
        self._folder_level = 0
        self._ul_started = 0

    
    def startElement(self, name, attrs):
        self._contents = ""
        
        if name == "xbel":
            self._inside = XBELHandler.ROOT
        elif name == "folder":
            self._inside = XBELHandler.FOLDER
            self._folder_level = self._folder_level + 1
        elif name == "bookmark":
            self._inside = XBELHandler.BOOKMARK
            self._url = attrs["href"]


    def characters(self, data):
        if self._inside != XBELHandler.NONE:
            self._contents = self._contents + data


    def endElement(self, name):
        if name == "title":
            if self._inside == XBELHandler.ROOT:
                self._out.write(top % (self._contents, self._contents))

            elif self._inside == XBELHandler.BOOKMARK:
                if not self._ul_started:
                    self._out.write("<ul>\n")
                    self._ul_started = 1
                self._out.write('<li><a href="%s">%s</a> \n' %
                                (self._url, self._contents))
            elif self._inside == XBELHandler.FOLDER:
                if self._ul_started == 1:
                    self._out.write("</ul>\n")
                    self._ul_started = 0
                
                if self._folder_level == 1:
                    self._out.write("<h2>%s</h2>\n" % self._contents)
                elif self._folder_level == 2:
                    self._out.write("<h3>%s</h3>\n" % self._contents)
                else:
    		    if not self._ul_started:
    			self._out.write("<ul>\n")
                    self._out.write("<li><b>%s</b>\n" % self._contents)
                    
                self._ul_started = 0


        elif name == "desc":
            if self._inside == XBELHandler.ROOT:
                self._out.write("<p>%s</p>" % self._contents)
                
        if name == "folder" or name == "bookmark":
            self._inside = XBELHandler.NONE

        if name == "folder":
            self._out.write("</ul>\n")
            self._folder_level = self._folder_level - 1


    def endDocument(self):
        self._out.write(bottom)        


# --- Main program

import sys

outf = codecs.open(sys.argv[2], "w", "iso-8859-1")
parse(sys.argv[1], XBELHandler(outf))
