
# --- State tracker

class SAXTracker(handler.ContentHandler):

    def __init__(self, keep_contents_of = []):
        self._elemstack = []
        self._contents = ""

        self._keep_contents_of = {}
        for element in keep_contents_of:
            self._keep_contents_of[element] = 1       

    def startElement(self, name, attrs):
        self._contents = ""
        self._elemstack.append(name)

    def characters(self, data):
        if self._keep_contents_of.has_key(self._elemstack[-1]):
            self._contents = self._contents + data
        
    def endElement(self, name):
        del self._elemstack[-1]


# --- ContentHandler

class XBELHandler(SAXTracker):
    
    def __init__(self, out):
        SAXTracker.__init__(self, ["title", "desc"])
        self._out = out # a generator object, not a file object
        self._url = None


    def startElement(self, name, attrs):
        if name == "bookmark":
            self._url = attrs["href"]

        SAXTracker.startElement(self, name, attrs)


    def endElement(self, name):
        SAXTracker.endElement(self, name)
        
        if name == "title":
            if self._elemstack[-1] == "xbel":
                self._out.top(self._contents)
            elif self._elemstack[-1] == "bookmark":
                self._out.bookmark(self._contents, self._url)
            elif self._elemstack[-1] == "folder":
                self._out.start_folder(self._contents)

        elif name == "desc" and self._elemstack[-1] == "xbel":
            self._out.description(self._contents)

        elif name == "folder":
            self._out.end_folder()


    def endDocument(self):
        self._out.bottom()


# --- HTMLGenerator

class HTMLGenerator:

    def __init__(self, out):
        self._out = out
        self._ul_started = 0
        self._folder_level = 0

    def top(self, title):
        self._out.write(html_top % (title, title))

    def start_folder(self, title):
        self._folder_level = self._folder_level + 1
        
        if self._ul_started == 1:
            self._out.write("</ul>\n")
            self._ul_started = 0

        if self._folder_level == 1:
            self._out.write("<h2>%s</h2>\n" % title)
        elif self._folder_level == 2:
            self._out.write("<h3>%s</h3>\n" % title)
        else:
            self._out.write("<li><b>%s</b>\n" % title)

        self._ul_started = 0

    def end_folder(self):
        self._out.write("</ul>\n")
        self._folder_level = self._folder_level - 1
        
    def bookmark(self, title, url):
        if not self._ul_started:
            self._out.write("<ul>\n")
            self._ul_started = 1
        self._out.write('<li><a href="%s">%s</a> \n' % (url, title))
        
    def description(self, text):
        self._out.write("<p>%s</p>" % text)        
        
    def bottom(self):
        self._out.write(html_btm)


# --- Main program

import sys

outf = codecs.open(sys.argv[2], "w", "iso-8859-1")
parse(sys.argv[1], XBELHandler(HTMLGenerator(outf)))

