# A SAX application that can build XBEL objects from XML documents.

from xml.sax import parse, handler
from saxtracker import SAXTracker # this is the SAXTracker class,
                                  # moved out into its own module.
from xbellib_model import *

# --- The SAX application

class XBELHandler(SAXTracker):

    def __init__(self):
        SAXTracker.__init__(self, ["title", "descr"])
        self._bookmarks = BookmarkCollection()
        self._current = self._bookmarks
        self._stack = []
        self._nodes_by_id = {}
        self._aliases = []


    def get_structure(self):
        return self._bookmarks

    
    def startElement(self, name, attrs):
        SAXTracker.startElement(self, name, attrs)

        node = None
        if name == "folder":
            node = Folder(attrs.get("id"))
            if attrs.has_key("folded"):
                node.set_is_folded(attrs["folded"] == "yes")
        elif name == "bookmark":
            node = Bookmark(attrs["href"], attrs.get("id"))
            node.set_added(attrs.get("added"))
            node.set_visited(attrs.get("visited"))
        elif name == "separator":
            node = Separator()
        elif name == "alias":
            node = Alias()
            self._aliases.append((node, attrs["ref"]))

        if node:
            self._current.add_node(node)
            self._stack.append(self._current)
            self._current = node

        if (name == "folder" or name == "bookmark") and \
           attrs.has_key("id"):
            self._nodes_by_id[attrs["id"]] = node

            
    def endElement(self, name):
        SAXTracker.endElement(self, name)
        
        if name == "title":
            self._current.set_title(self._contents)
        elif name == "descr":
            self._current.set_descr(self._contents)

        elif name in ("folder", "bookmark", "separator", "alias"):
            self._current = self._stack.pop()

            
    def endDocument(self):
        for (alias, ref) in self._aliases:
            alias.set_ref_node(self._nodes_by_id.get(ref))

            
# --- utility functions

def load_xbel(filename, error_handler = None):
    handler = XBELHandler()
    parse(filename, handler, error_handler)
    return handler.get_structure()

