# A function for writing out XBEL objects as documents.

from xbellib_model import *
import xmlgen

def serialize(xbel, out):
    serialize_to_generator(xbel, xmlgen.fileobj_writer(out))

def serialize_to_generator(xbel, gen):
    gen.startElement("xbel")
    _serialize_title(xbel, gen)
    _serialize_nodes(xbel.get_nodes(), gen)
    gen.endElement()
    
def _serialize_title(node, gen):
    gen.fieldElementOptional("title", node.get_title())
    gen.fieldElementOptional("descr", node.get_descr())
    
def _serialize_nodes(nodes, gen):
    for node in nodes:
        if isinstance(node, Folder):
            attrs = {}
            if node.get_id():
                attrs["id"] = node.get_id()
            attrs["folded"] = {1 : "yes", 0 : "no"}[node.is_folded()]
            xmlgen.startElement("folder", attrs)
            _serialize_title(node, gen)
            _serialize_nodes(node.get_nodes(), gen)
            xmlgen.endElement()

        elif isinstance(node, Bookmark):
            attrs = {"url" : node.get_url()}
            if node.get_id():
                attrs["id"] = node.get_id()
            if node.get_added():
                attrs["added"] = node.get_added()
            if node.get_visited():
                attrs["visited"] = node.get_visited()
            xmlgen.startElement("bookmark", attrs)
            
            _serialize_title(node, gen)
            xmlgen.endElement()

        elif isinstance(node, Separator):
            xmlgen.emptyElement("separator")

        elif isinstance(node, Alias):
            xmlgen.emptyElement("alias",
                                {"ref" : node.get_ref_node().get_id()})
