# A set of Python classes for representing XBEL documents.

class NodeWithTitle:
    """Common abstract base class for xbel, folder and bookmark
    elements."""

    def __init__(self, id = None, title = None, descr = None):
        self._id    = id
        self._title = title
        self._descr = descr
        self._added = None

    def get_id(self):
        return self._id

    def set_id(self, id):
        self._id = id
        
    def get_title(self):
        return self._title

    def set_title(self, title):
        self._title = title
        
    def get_descr(self):
        return self._descr

    def set_descr(self, descr):
        self._descr = descr

    def get_added(self):
        return self._added

    def set_added(self, added):
        self._added = added

        
class NodeWithChildren(NodeWithTitle):
    "Common abstract base class for xbel and folder elements."
        
    def __init__(self, id = None, title = None, descr = None):
        NodeWithTitle.__init__(self, id, title, descr)
        self._nodes = []
        
    def get_nodes(self):
        return self._nodes

    def add_node(self, node):
        self._nodes.append(node)
        
        
class BookmarkCollection(NodeWithChildren):
    "Represents an entire XBEL document."


class Folder(NodeWithChildren):
    "Represents a folder element."

    def __init__(self, id = None, title = None, descr = None):
        NodeWithChildren.__init__(self, id, title, descr)
        self._folded = 1

    def is_folded(self):
        return self._folded

    def set_is_folded(self, folded):
        self._folded = folded
    
class Bookmark(NodeWithTitle):
    "Represents a bookmark element."

    def __init__(self, url, id = None, title = None, descr = None):
        NodeWithTitle.__init__(self, id, title, descr)
        self._url = url
        self._visited = None
        self._modified = None

    def get_url(self):
        return self._url

    def set_url(self, url):
        self._url = url

    def get_visited(self):
        return self._visited

    def set_visited(self, visited):
        self._visited = visited

    def get_modified(self):
        return self._modified

    def set_modified(self, modified):
        self._modified = modified

    
class Separator:
    "Represents separator elements."


class Alias:
    "Represents alias elements."

    def __init__(self, ref_node = None):
        self._ref_node = ref_node
        
    def get_ref_node(self):
        return self._ref_node

    def set_ref_node(self, ref_node):
        self._ref_node = ref_node
