#!/usr/bin/python

import sgmllib, urllib, string, re, anydbm, smtplib, sys, time, pickle

# Object structure

class Document:

    def __init__(self, id, url, title, date, other):
        self._id = id or url
        self._url = url
        self._title = string.join(string.split(title), " ")
        self._date = date
        self._other = string.join(string.split(other), " ")

    def get_id(self):
        return self._id

    def get_url(self):
        return self._url

    def get_title(self):
        return self._title

    def get_date(self):
        return self._date

    def get_other(self):
        return self._other
    
    def __repr__(self):
        return "<W3C Document %s, '%s'>" % (self._id, self._title)

# Config

datfile    = "specs.dat"
notify     = ["larsga@garshol.priv.no"]
from_addr  = "larsga@garshol.priv.no"
smtpserver = "mail.garshol.priv.no"
send_mail  = 0

# Spec page parser

reg_date = re.compile("\d\d? \w+ \d+")

class SpecParser(sgmllib.SGMLParser):

    from htmlentitydefs import entitydefs

    def __init__(self):
        sgmllib.SGMLParser.__init__(self, None)
        self._specs = []
        self._in_middle = 0
        self._inside = None
        self._parent = None

        self._name   = None
        self._href   = None
        self._title  = ""
        self._date   = ""
        self._other  = ""

    def start_dt(self, attrs):
        if self._in_middle:
            self._inside = "dt"
        
    def start_a(self, attrs):
        if not self._in_middle:
            self._in_middle = \
                self._getattr("name", attrs) == "Recommendations"
                    
            return
        else:
            return

        if self._getattr("name", attrs) == "About":
            self._in_middle = 0
            return            
        
        if self._inside in ("dt", "li"):
            self._href = self._getattr("href", attrs)
            self._name = self._getattr("name", attrs)
            self._parent = self._inside
            self._inside = "a"
        
    def _getattr(self, attr, attrs):
        for (a, v) in attrs:
            if a == attr:
                return v

        return None

    def start_dd(self,attrs):
        if self._in_middle and self._inside == "dt":
            self._inside = "dd"

    def start_ul(self,attrs):
        if self._in_middle:
            self._inside = "ul"

    def start_li(self, attrs):
        if self._in_middle and self._inside == "ul":
            self._inside = "li"
            
    def handle_data(self,text):
        if self._in_middle:
            if self._inside == "dd":
                self._date = self._date + text
            elif self._inside == "a":
                self._title = self._title + text
            elif self._inside == "li":
                self._other = self._other + text
                
    def end_a(self):
        if self._in_middle and self._inside == "a":
            self._inside = self._parent
            self._parent = None
            
    def end_dd(self):
        if self._in_middle and self._inside == "dd" and self._title:
            match = reg_date.match(self._date)            
            date = match.group()
            other = self._date[match.end()+1 : ]
            self._title = string.strip(self._title)

            self._specs.append(Document(self._name, self._href, self._title,
                                        date, other))

            self._inside = ""
            self._date   = ""
            self._title  = ""

    def end_li(self):
        if self._in_middle and self._inside == "li":
            date = string.strip(self._date)
            other = self._other                
            self._title = string.strip(self._title)

            self._specs.append(Document(self._name, self._href, self._title,
                                        date, other))

            self._inside = "ul"
            self._title  = ""
            self._other  = ""

def check_for_changes():
    # load current specs
    specs = parse(urllib.urlopen("http://www.w3.org/TR/"))
    #specs = parse(open(sys.argv[1]))
    newspecs = {}    
    for spec in specs:
        newspecs[spec.get_id()] = spec

    # load data from previous run
    try:
        inf = open(datfile, "rb")
    except IOError, e:
        if e.errno == 2:
            # if datfile does not exist, just save and exit
            savedata(newspecs)
            return

        raise # some other problem; report it
    
    p = pickle.Unpickler(inf)
    oldspecs = p.load()
    inf.close()

    # save current data
    savedata(newspecs)

    # Compare!
    import StringIO
    out = StringIO.StringIO()

    for spec in newspecs.values():
        if oldspecs.has_key(spec.get_id()):
            old = oldspecs[spec.get_id()]
            if spec.get_date() != old.get_date():
                out.write("Spec '%s' updated\n" % spec.get_title())
        else:
            out.write("New spec '%s' at '%s'\n" % (spec.get_title(),
                                                   spec.get_url()))

    report = out.getvalue()

    # Sending email

    msgtemp = \
    """Subject: watchw3 update
    From: %s (W3C-Watch mailer)

    %s
    --
    watchw3"""

    if report != "":
        if send_mail:
            smtp=smtplib.SMTP(smtpserver)
            smtp.sendmail(from_addr, notify, msgtemp % (from_addr, report))
        else:
            print report

def savedata(specs):
    outf = open(datfile, "wb")
    p = pickle.Pickler(outf)
    p.dump(specs)
    outf.close()
            
def parse(inf):
    parser = SpecParser()
    while 1:
        data = inf.read(16384)
        if data == "":
            break

        parser.feed(data)

    parser.close()

    return parser._specs
            
def debug_dump(specs):
    print "<TABLE>"
    for spec in specs:
        print "<TR><TD>%s <TD>%s <TD>%s <TD>%s <TD>%s" % (spec.get_id(),
                                                          spec.get_title(),
                                                          spec.get_url(),
                                                          spec.get_date(),
                                                          spec.get_other())
    print "<TABLE>"


# --- Main program

if __name__ == "__main__":
    #debug_dump(parse(open(sys.argv[1])))
    check_for_changes()
