        
import string, os
import xml.dom.ext
from xml.dom.Node import Node
from xml.xslt import XsltElement, XSL_NAMESPACE
from xml.xpath import XPathParser
from xml.xpath.CoreFunctions import String

extension_uri = 'http://garshol.priv.no/symbolic/'
varname = (extension_uri, 'items')


# --- lmg:group

class GroupElement(XsltElement):

    legalAttrs = ('select', 'group-by')
    
    def setup(self):
        self.__dict__['_nss'] = xml.dom.ext.GetAllNs(self)
        parser = XPathParser.XPathParser()
        group_by = self.getAttributeNS('', 'group-by')
        select = self.getAttributeNS('', 'select')
        if not group_by:
            raise Exception("'group-by' attribute on <group> required!")
        if not select:
            raise Exception("'select' attribute on <group> required!")

        
        self.__dict__['_group_by'] = parser.parseExpression(group_by)
        self.__dict__['_select'] = parser.parseExpression(select)

        self.__dict__['_sort_specs'] = []
        for child in self.childNodes:
            if (child.namespaceURI, child.localName) == \
               (XSL_NAMESPACE, 'sort'):
                self._sort_specs.append(child)

                
    def instantiate(self, context, processor):
        orig_state = context.copy()
        context.setNamespaces(self._nss)

        # select the nodes and group them
        groups = {}
        for node in self._select.evaluate(context):
            context.node = node
            key = String(context, self._group_by.evaluate(context))

            try:
                groups[key].append(node)
            except KeyError:
                groups[key] = [node]

               
        # sort the groups
        nodelist = []
        groupmap = {}
        for (key, group_nodes) in groups.items():
            nodelist.append(group_nodes[0])
            groupmap[id(group_nodes[0])] = groups[key]
            
        if len(groups) > 1 and self._sort_specs:
            nodelist = self._sort_specs[0].instantiate(context,
                                           processor,
                                           nodelist,
                                           self._sort_specs[1 : ])[1]
            groups = []
            for node in nodelist:
                groups.append(groupmap[id(node)])
        else:
            groups = groups.values()

            
        # do the instantiation
        context.size = len(groups)
        for ix in range(len(groups)):
            items = groups[ix]
            context.node = items[0]
            context.position = ix + 1
            context.varBindings[varname] = items
            for child in self.childNodes:
                context = child.instantiate(context, processor)[0]

                
        context.set(orig_state)
        return (context,)

# --- lmg:item

class ItemElement(XsltElement):

    def setup(self):
        self.__dict__['_nss'] = xml.dom.ext.GetAllNs(self)

        self.__dict__['_sort_specs'] = []
        for child in self.childNodes:
            if (child.namespaceURI, child.localName) == \
               (XSL_NAMESPACE, 'sort'):
                self._sort_specs.append(child)
        
    def instantiate(self, context, processor):
        orig_state = context.copy()
        context.setNamespaces(self._nss)

        items = context.varBindings[varname]
        
        # sort the items
        if len(items) > 1 and self._sort_specs:
            items = self._sort_specs[0].instantiate(context, processor,
                                             items,
                                             self._sort_specs[1 : ])[1]

            
        # do instantiation
        context.size = len(items)
        for ix in range(len(items)):
            item = items[ix]
            context.node = item
            context.position = ix + 1
            for child in self.childNodes:
                context = child.instantiate(context, processor)[0]
                
        context.set(orig_state)
        return (context,)

# --- Mapping dictionary
    
ExtElements = {
    (extension_uri, 'group'): GroupElement,
    (extension_uri, 'item'):  ItemElement
    }
