
from xml.sax.handler import ContentHandler
from xml.dom.ext.reader.Sax import Reader
from xml.xpath import Compile, CreateContext
from xml.xpath.CoreFunctions import String

# ===== OBJECT STRUCTURE

class ObjectMapping:

    def __init__(self, context = None):
        if context:
            context = Compile(context)
        self._context = context
            
        self._fields = []

    def add_field(self, field):
        self._fields.append(field)

    def get_context_node(self, current):
        if self._context:
            value = self._context.evaluate(CreateContext(current))
            if type(value) == type([]):
                value = value[0]
            return value
        else:
            return current

    def get_fields(self):
        return self._fields


class BasicField:

    def __init__(self, name, value):
        self._name = name
        self._value = Compile(value)

    def get_name(self):
        return self._name

    def get_value(self, context):
        value = self._value.evaluate(context)
        if type(value) == type([]):
            value = String(CreateContext(value))
        return value


class ComplexField:

    def __init__(self, name):
        self._name = name

    def set_type(self, type):
        self._type = type

    def set_select(self, select):
        self._select = Compile(select)

    def set_object(self, object):
        self._object = object

    def get_name(self):
        return self._name

    def get_value(self, context):
        objs = []
        for node in self._select.evaluate(context):
            obj = build_objects(self._object, node)
            objs.append(obj)

        return objs



# ===== BUILDER

class MappingBuilder(ContentHandler):

    def __init__(self):
        self._objstack = []
        self._curfield = None

    def get_root_object(self):
        return self._objstack[-1]

    def startElement(self, name, attrs):
        if name == "object":
            self._objstack.append(ObjectMapping(attrs.get("context")))

            if self._curfield:
                self._curfield.set_object(self._objstack[-1])

        elif name == "field":
            if attrs.has_key("value"):
                field = BasicField(attrs.get("name"), attrs.get("value"))
            else:
                field = ComplexField(attrs.get("name"))
                self._curfield = field

            self._objstack[-1].add_field(field)

        elif name == "list":
            self._curfield.set_type("list")
            self._curfield.set_select(attrs.get("select"))
            

    def endElement(self, name):
        if name == "object" and len(self._objstack) > 1:
            del self._objstack[-1]



# ===== INTERPRETER

class DataObject:
    pass

def build_objects(mapping, document):
    context = CreateContext(mapping.get_context_node(document))
    object = DataObject()
    for field in mapping.get_fields():
        name = field.get_name()
        value = field.get_value(context)
        
        setattr(object, name, value)

    return object



# ===== UTILITIES

def load_mapping(file):
    from xml.sax import make_parser
    
    builder = MappingBuilder()
    parser = make_parser()
    parser.setContentHandler(builder)
    parser.parse(file)

    return builder.get_root_object()
    
def load_objects(mapping, file):
    document = Reader().fromStream(open(file))
    return build_objects(mapping, document)

